"""Sentiment analysis tools"""

import json
from dol import Pipe

from mood.util import add_egress


@add_egress(json.loads)
def sentiment_analysis(headlines):
    """Get sentiment analysis for headlines."""
    from oa import ask

    if not isinstance(headlines, str):  # Pattern: ingress
        headlines = "\n\n".join(
            headlines
        )  # join with two newlines to separate headlines
    headline_sentiments = ask.ai.sentiment_analysis(headlines)
    return headline_sentiments



"""
Utils for mood
"""

import os
from functools import wraps, partial

from i2 import get_app_data_folder

import dol

pkg_name = "mood"

_root_app_data_dir = get_app_data_folder()
app_data_dir = os.environ.get(
    f"{pkg_name.upper()}_APP_DATA_DIR",
    os.path.join(_root_app_data_dir, pkg_name),
)
app_data_dir = dol.ensure_dir(app_data_dir, verbose=f"Making app dir: {app_data_dir}")
djoin = partial(os.path.join, app_data_dir)


# NOte: Other versions: i2.postprocess and possible with partial of i2.wrap
#   Put a simple version of this functionality here to grow deps slowly.
#   If needs to be maintained, don't: Just source it from i2.
def add_egress(post):
    """Decorator to add an egress to the output of a function.

    >>> @add_egress(str)
    ... def foo(x: int) -> int:
    ...     return x + 1
    >>> foo(3)
    '4'

    Note that if you need to access the underlying wrapped function, you can do so in
    the python-standard way:

    >>> underlying_foo = foo.__wrapped__
    >>> underlying_foo(3)
    4
    """

    def _add_egress(func):
        @wraps(func)
        def func_with_added_egress(*args, **kwargs):
            output = func(*args, **kwargs)
            return post(output)

        return func_with_added_egress

    return _add_egress



"""A script that searches for news and saves the results to a file.

Meant to be used in a cron on a server to periodically search for news.

"""

from mood.tools import search_and_save_news
import argh


def main():
    argh.dispatch_command(search_and_save_news)


if __name__ == "__main__":
    main()



"""

Sentiment analysis for stock market data.

>>> from mood import headlines_mood
>>> headlines_mood()  # doctest: +SKIP
{"Yaccarino shakes up X amid Musk's pressure on costs, FT says": -3,
 'Coup-hit Niger was betting on a China-backed oil pipeline as a lifeline. Then the troubles began': -7,
 'A Mexico City neighborhood keeps the iconic Volkswagen Beetle alive': 1,
 ...
 "Here's the Average Social Security Benefit at Age 62 -- and Why It's Not the Best News for Retirees": -5,
 'Analyst Report: Mitsubishi UFJ Financial Group, Inc.': 0,
 'Forget NextEra Energy. Buy This Magnificent Dividend King Instead': 6}
"""

from mood.sentiment import sentiment_analysis
from mood.tools import headlines_mood, headlines, search_and_save_news



"""Tools for sentiment analysis"""

from dol import Pipe
from scoopy.data_sources import headlines
from mood.sentiment import sentiment_analysis


headlines_mood = Pipe(headlines, sentiment_analysis)


# --------------------------------------------------------------------------------------
# Search and save news

from functools import partial
from typing import Union, Iterable
import os
import re
from dol import JsonFiles, mk_dirs_if_missing
from scoopy import search_news, yahoo_finance_news_search
from mood.util import djoin

filename_safe_pattern = re.compile(r"[^a-zA-Z0-9_\-]")


def mk_filename_safe(string: str) -> str:
    return filename_safe_pattern.sub("_", string)


def current_time_str(format="%Y-%m-%d--%H-%M-%S"):
    from datetime import datetime

    now = datetime.now()
    return now.strftime(format)


def get_key(query, prefix="", suffix="", *, day_folder=False):
    if day_folder:
        day_folder_str = current_time_str("%Y-%m-%d") + os.path.sep
    else:
        day_folder_str = ""
    now_string_seconds = current_time_str("%Y-%m-%d--%H-%M-%S")
    query_str = mk_filename_safe(query)
    return f"{prefix}{day_folder_str}{now_string_seconds}__{query_str}{suffix}"


DFLT_STORE = djoin("news", "searches")
json_file_key = partial(get_key, suffix=".json", day_folder=True)


def _search_and_save_news(
    query, store=DFLT_STORE, *, search_func=search_news, query_to_key=json_file_key
):

    if isinstance(store, str):
        import os

        rootdir = os.path.expanduser(store)
        # create the rootdir if it does not exist
        os.makedirs(rootdir, exist_ok=True)

        store = mk_dirs_if_missing(JsonFiles(rootdir))

    results = search_func(query)

    key = query_to_key(query)
    store[key] = results
    return key


_newsdata_search_default_params = {
    "qInTitle": None,  # Optional: Use if you want keywords in headlines only
    "category": "business, technology, politics",  # Focus on market-relevant categories
    "country": "us,gb,cn,jp,de",  # Major financial hubs
    "language": "en",
    # "timeframe": "24",  # Last 24 hours (Requires premium plan)
    # "prioritydomain": "newsdata.io"  # Optional: Prioritize high-quality sources
}

search_news_from = {
    "newsdata": partial(
        search_news, source="newsdata", **_newsdata_search_default_params
    ),
    "yahoo_finance": partial(search_news, source="yahoo_finance"),
    "yahoo_finance_headlines": partial(search_news, source="yahoo_finance_headlines"),
}


def _probably_a_file(string):
    """Check if the string should be considered as a file"""
    if isinstance(string, str) and os.path.isfile(string):
        if (
            string.endswith(".json")
            or string[-4:] in {".txt", ".csv", ".tsv"}
            or os.path.sep in string
        ):
            return True

    return False


def search_and_save_news(
    query: Union[str, Iterable[str]] = "",
    source: str = "yahoo_finance_headlines",
    *,
    verbose: bool = False,
    store=DFLT_STORE,
):
    assert source in search_news_from, f"Unknown news source: {source}"
    prefix = source + os.path.sep
    _search_func = search_news_from[source]
    _file_key = partial(get_key, prefix=prefix, suffix=".json", day_folder=True)

    if isinstance(query, str):
        if _probably_a_file(query):
            if query.endswith(".json"):
                with open(query) as f:
                    query = json.load(f)
            else:
                with open(query) as f:
                    query = f.read().strip().split("\n")
        else:
            query = [query]

    if verbose:
        print(f"Searching and saving news in {store}")

    for q in query:
        t = _search_and_save_news(
            q, search_func=_search_func, query_to_key=_file_key, store=store
        )
        if verbose:
            print(f"{t}")
